# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::DataMigration::Mgmt::V2018_07_15_preview
  #
  # Data Migration Client
  #
  class Files
    include MsRestAzure

    #
    # Creates and initializes a new instance of the Files class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [DataMigrationServiceClient] reference to the DataMigrationServiceClient
    attr_reader :client

    #
    # Get files in a project
    #
    # The project resource is a nested resource representing a stored migration
    # project. This method returns a list of files owned by a project resource.
    #
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<ProjectFile>] operation results.
    #
    def list(group_name, service_name, project_name, custom_headers:nil)
      first_page = list_as_lazy(group_name, service_name, project_name, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Get files in a project
    #
    # The project resource is a nested resource representing a stored migration
    # project. This method returns a list of files owned by a project resource.
    #
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_with_http_info(group_name, service_name, project_name, custom_headers:nil)
      list_async(group_name, service_name, project_name, custom_headers:custom_headers).value!
    end

    #
    # Get files in a project
    #
    # The project resource is a nested resource representing a stored migration
    # project. This method returns a list of files owned by a project resource.
    #
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_async(group_name, service_name, project_name, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'group_name is nil' if group_name.nil?
      fail ArgumentError, 'service_name is nil' if service_name.nil?
      fail ArgumentError, 'project_name is nil' if project_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{groupName}/providers/Microsoft.DataMigration/services/{serviceName}/projects/{projectName}/files'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'groupName' => group_name,'serviceName' => service_name,'projectName' => project_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataMigration::Mgmt::V2018_07_15_preview::Models::FileList.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Get file information
    #
    # The files resource is a nested, proxy-only resource representing a file
    # stored under the project resource. This method retrieves information about a
    # file.
    #
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ProjectFile] operation results.
    #
    def get(group_name, service_name, project_name, file_name, custom_headers:nil)
      response = get_async(group_name, service_name, project_name, file_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Get file information
    #
    # The files resource is a nested, proxy-only resource representing a file
    # stored under the project resource. This method retrieves information about a
    # file.
    #
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(group_name, service_name, project_name, file_name, custom_headers:nil)
      get_async(group_name, service_name, project_name, file_name, custom_headers:custom_headers).value!
    end

    #
    # Get file information
    #
    # The files resource is a nested, proxy-only resource representing a file
    # stored under the project resource. This method retrieves information about a
    # file.
    #
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(group_name, service_name, project_name, file_name, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'group_name is nil' if group_name.nil?
      fail ArgumentError, 'service_name is nil' if service_name.nil?
      fail ArgumentError, 'project_name is nil' if project_name.nil?
      fail ArgumentError, 'file_name is nil' if file_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{groupName}/providers/Microsoft.DataMigration/services/{serviceName}/projects/{projectName}/files/{fileName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'groupName' => group_name,'serviceName' => service_name,'projectName' => project_name,'fileName' => file_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataMigration::Mgmt::V2018_07_15_preview::Models::ProjectFile.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Create a file resource
    #
    # The PUT method creates a new file or updates an existing one.
    #
    # @param parameters [ProjectFile] Information about the file
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ProjectFile] operation results.
    #
    def create_or_update(parameters, group_name, service_name, project_name, file_name, custom_headers:nil)
      response = create_or_update_async(parameters, group_name, service_name, project_name, file_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Create a file resource
    #
    # The PUT method creates a new file or updates an existing one.
    #
    # @param parameters [ProjectFile] Information about the file
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def create_or_update_with_http_info(parameters, group_name, service_name, project_name, file_name, custom_headers:nil)
      create_or_update_async(parameters, group_name, service_name, project_name, file_name, custom_headers:custom_headers).value!
    end

    #
    # Create a file resource
    #
    # The PUT method creates a new file or updates an existing one.
    #
    # @param parameters [ProjectFile] Information about the file
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def create_or_update_async(parameters, group_name, service_name, project_name, file_name, custom_headers:nil)
      fail ArgumentError, 'parameters is nil' if parameters.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'group_name is nil' if group_name.nil?
      fail ArgumentError, 'service_name is nil' if service_name.nil?
      fail ArgumentError, 'project_name is nil' if project_name.nil?
      fail ArgumentError, 'file_name is nil' if file_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::DataMigration::Mgmt::V2018_07_15_preview::Models::ProjectFile.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{groupName}/providers/Microsoft.DataMigration/services/{serviceName}/projects/{projectName}/files/{fileName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'groupName' => group_name,'serviceName' => service_name,'projectName' => project_name,'fileName' => file_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:put, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 201
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataMigration::Mgmt::V2018_07_15_preview::Models::ProjectFile.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end
        # Deserialize Response
        if status_code == 201
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataMigration::Mgmt::V2018_07_15_preview::Models::ProjectFile.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Delete file
    #
    # This method deletes a file.
    #
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    #
    def delete(group_name, service_name, project_name, file_name, custom_headers:nil)
      response = delete_async(group_name, service_name, project_name, file_name, custom_headers:custom_headers).value!
      nil
    end

    #
    # Delete file
    #
    # This method deletes a file.
    #
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def delete_with_http_info(group_name, service_name, project_name, file_name, custom_headers:nil)
      delete_async(group_name, service_name, project_name, file_name, custom_headers:custom_headers).value!
    end

    #
    # Delete file
    #
    # This method deletes a file.
    #
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def delete_async(group_name, service_name, project_name, file_name, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'group_name is nil' if group_name.nil?
      fail ArgumentError, 'service_name is nil' if service_name.nil?
      fail ArgumentError, 'project_name is nil' if project_name.nil?
      fail ArgumentError, 'file_name is nil' if file_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{groupName}/providers/Microsoft.DataMigration/services/{serviceName}/projects/{projectName}/files/{fileName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'groupName' => group_name,'serviceName' => service_name,'projectName' => project_name,'fileName' => file_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:delete, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 204
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?

        result
      end

      promise.execute
    end

    #
    # Update a file
    #
    # This method updates an existing file.
    #
    # @param parameters [ProjectFile] Information about the file
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [ProjectFile] operation results.
    #
    def update(parameters, group_name, service_name, project_name, file_name, custom_headers:nil)
      response = update_async(parameters, group_name, service_name, project_name, file_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Update a file
    #
    # This method updates an existing file.
    #
    # @param parameters [ProjectFile] Information about the file
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def update_with_http_info(parameters, group_name, service_name, project_name, file_name, custom_headers:nil)
      update_async(parameters, group_name, service_name, project_name, file_name, custom_headers:custom_headers).value!
    end

    #
    # Update a file
    #
    # This method updates an existing file.
    #
    # @param parameters [ProjectFile] Information about the file
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def update_async(parameters, group_name, service_name, project_name, file_name, custom_headers:nil)
      fail ArgumentError, 'parameters is nil' if parameters.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'group_name is nil' if group_name.nil?
      fail ArgumentError, 'service_name is nil' if service_name.nil?
      fail ArgumentError, 'project_name is nil' if project_name.nil?
      fail ArgumentError, 'file_name is nil' if file_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::DataMigration::Mgmt::V2018_07_15_preview::Models::ProjectFile.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{groupName}/providers/Microsoft.DataMigration/services/{serviceName}/projects/{projectName}/files/{fileName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'groupName' => group_name,'serviceName' => service_name,'projectName' => project_name,'fileName' => file_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:patch, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataMigration::Mgmt::V2018_07_15_preview::Models::ProjectFile.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Request storage information for downloading the file content
    #
    # This method is used for requesting storage information using which contents
    # of the file can be downloaded.
    #
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [FileStorageInfo] operation results.
    #
    def read(group_name, service_name, project_name, file_name, custom_headers:nil)
      response = read_async(group_name, service_name, project_name, file_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Request storage information for downloading the file content
    #
    # This method is used for requesting storage information using which contents
    # of the file can be downloaded.
    #
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def read_with_http_info(group_name, service_name, project_name, file_name, custom_headers:nil)
      read_async(group_name, service_name, project_name, file_name, custom_headers:custom_headers).value!
    end

    #
    # Request storage information for downloading the file content
    #
    # This method is used for requesting storage information using which contents
    # of the file can be downloaded.
    #
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def read_async(group_name, service_name, project_name, file_name, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'group_name is nil' if group_name.nil?
      fail ArgumentError, 'service_name is nil' if service_name.nil?
      fail ArgumentError, 'project_name is nil' if project_name.nil?
      fail ArgumentError, 'file_name is nil' if file_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{groupName}/providers/Microsoft.DataMigration/services/{serviceName}/projects/{projectName}/files/{fileName}/read'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'groupName' => group_name,'serviceName' => service_name,'projectName' => project_name,'fileName' => file_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataMigration::Mgmt::V2018_07_15_preview::Models::FileStorageInfo.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Request information for reading and writing file content.
    #
    # This method is used for requesting information for reading and writing the
    # file content.
    #
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [FileStorageInfo] operation results.
    #
    def read_write(group_name, service_name, project_name, file_name, custom_headers:nil)
      response = read_write_async(group_name, service_name, project_name, file_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Request information for reading and writing file content.
    #
    # This method is used for requesting information for reading and writing the
    # file content.
    #
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def read_write_with_http_info(group_name, service_name, project_name, file_name, custom_headers:nil)
      read_write_async(group_name, service_name, project_name, file_name, custom_headers:custom_headers).value!
    end

    #
    # Request information for reading and writing file content.
    #
    # This method is used for requesting information for reading and writing the
    # file content.
    #
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param file_name [String] Name of the File
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def read_write_async(group_name, service_name, project_name, file_name, custom_headers:nil)
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'group_name is nil' if group_name.nil?
      fail ArgumentError, 'service_name is nil' if service_name.nil?
      fail ArgumentError, 'project_name is nil' if project_name.nil?
      fail ArgumentError, 'file_name is nil' if file_name.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{groupName}/providers/Microsoft.DataMigration/services/{serviceName}/projects/{projectName}/files/{fileName}/readwrite'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'groupName' => group_name,'serviceName' => service_name,'projectName' => project_name,'fileName' => file_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataMigration::Mgmt::V2018_07_15_preview::Models::FileStorageInfo.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Get files in a project
    #
    # The project resource is a nested resource representing a stored migration
    # project. This method returns a list of files owned by a project resource.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [FileList] operation results.
    #
    def list_next(next_page_link, custom_headers:nil)
      response = list_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Get files in a project
    #
    # The project resource is a nested resource representing a stored migration
    # project. This method returns a list of files owned by a project resource.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_next_with_http_info(next_page_link, custom_headers:nil)
      list_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Get files in a project
    #
    # The project resource is a nested resource representing a stored migration
    # project. This method returns a list of files owned by a project resource.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DataMigration::Mgmt::V2018_07_15_preview::Models::FileList.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Get files in a project
    #
    # The project resource is a nested resource representing a stored migration
    # project. This method returns a list of files owned by a project resource.
    #
    # @param group_name [String] Name of the resource group
    # @param service_name [String] Name of the service
    # @param project_name [String] Name of the project
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [FileList] which provide lazy access to pages of the response.
    #
    def list_as_lazy(group_name, service_name, project_name, custom_headers:nil)
      response = list_async(group_name, service_name, project_name, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

  end
end
